'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var umi = require('@metaplex-foundation/umi');
var errors = require('./errors.cjs');
var getSizeDescription = require('./getSizeDescription.cjs');

function bytes(options = {}) {
  const size = options.size ?? 'variable';
  const description = options.description ?? `bytes(${getSizeDescription.getSizeDescription(size)})`;
  const byteSerializer = {
    description,
    fixedSize: null,
    maxSize: null,
    serialize: value => new Uint8Array(value),
    deserialize: (bytes, offset = 0) => {
      const slice = new Uint8Array(bytes.slice(offset));
      return [slice, offset + slice.length];
    }
  };
  if (size === 'variable') {
    return byteSerializer;
  }
  if (typeof size === 'number') {
    return umi.fixSerializer(byteSerializer, size, description);
  }
  return {
    description,
    fixedSize: null,
    maxSize: null,
    serialize: value => {
      const contentBytes = byteSerializer.serialize(value);
      const lengthBytes = size.serialize(contentBytes.length);
      return umi.mergeBytes([lengthBytes, contentBytes]);
    },
    deserialize: (buffer, offset = 0) => {
      if (buffer.slice(offset).length === 0) {
        throw new errors.DeserializingEmptyBufferError('bytes');
      }
      const [lengthBigInt, lengthOffset] = size.deserialize(buffer, offset);
      const length = Number(lengthBigInt);
      offset = lengthOffset;
      const contentBuffer = buffer.slice(offset, offset + length);
      if (contentBuffer.length < length) {
        throw new errors.NotEnoughBytesError('bytes', length, contentBuffer.length);
      }
      const [value, contentOffset] = byteSerializer.deserialize(contentBuffer);
      offset += contentOffset;
      return [value, offset];
    }
  };
}

exports.bytes = bytes;
//# sourceMappingURL=bytes.cjs.map
