'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var errors = require('./errors.cjs');
var sumSerializerSizes = require('./sumSerializerSizes.cjs');

function getResolvedSize(size, childrenSizes, bytes, offset) {
  if (typeof size === 'number') {
    return [size, offset];
  }
  if (typeof size === 'object') {
    return size.deserialize(bytes, offset);
  }
  if (size === 'remainder') {
    const childrenSize = sumSerializerSizes.sumSerializerSizes(childrenSizes);
    if (childrenSize === null) {
      throw new errors.BeetSerializerError('Serializers of "remainder" size must have fixed-size items.');
    }
    const remainder = bytes.slice(offset).length;
    if (remainder % childrenSize !== 0) {
      throw new errors.BeetSerializerError(`Serializers of "remainder" size must have a remainder that is a multiple of its item size. ` + `Got ${remainder} bytes remaining and ${childrenSize} bytes per item.`);
    }
    return [remainder / childrenSize, offset];
  }
  throw new errors.BeetSerializerError(`Unknown size type: ${JSON.stringify(size)}.`);
}

exports.getResolvedSize = getResolvedSize;
//# sourceMappingURL=getResolvedSize.cjs.map
