'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var umi = require('@metaplex-foundation/umi');
var errors = require('./errors.cjs');
var getSizeDescription = require('./getSizeDescription.cjs');
var sumSerializerSizes = require('./sumSerializerSizes.cjs');
var numbers = require('./numbers.cjs');

function option(item, options = {}) {
  const prefix = options.prefix ?? numbers.u8();
  const fixed = options.fixed ?? false;
  let descriptionSuffix = `; ${getSizeDescription.getSizeDescription(prefix)}`;
  let fixedSize = item.fixedSize === 0 ? prefix.fixedSize : null;
  if (fixed) {
    if (item.fixedSize === null || prefix.fixedSize === null) {
      throw new errors.BeetSerializerError('Fixed options can only be used with fixed-size serializers');
    }
    descriptionSuffix += '; fixed';
    fixedSize = prefix.fixedSize + item.fixedSize;
  }
  return {
    description: options.description ?? `option(${item.description + descriptionSuffix})`,
    fixedSize,
    maxSize: sumSerializerSizes.sumSerializerSizes([prefix.maxSize, item.maxSize]),
    serialize: optionOrNullable => {
      const option = umi.isOption(optionOrNullable) ? optionOrNullable : umi.wrapNullable(optionOrNullable);
      const prefixByte = prefix.serialize(Number(umi.isSome(option)));
      if (fixed) {
        const itemFixedSize = item.fixedSize;
        const itemBytes = umi.isSome(option) ? item.serialize(option.value).slice(0, itemFixedSize) : new Uint8Array(itemFixedSize).fill(0);
        return umi.mergeBytes([prefixByte, itemBytes]);
      }
      const itemBytes = umi.isSome(option) ? item.serialize(option.value) : new Uint8Array();
      return umi.mergeBytes([prefixByte, itemBytes]);
    },
    deserialize: (bytes, offset = 0) => {
      if (bytes.slice(offset).length === 0) {
        throw new errors.DeserializingEmptyBufferError('option', umi.none());
      }
      const fixedOffset = offset + (prefix.fixedSize ?? 0) + (item.fixedSize ?? 0);
      const [isSome, prefixOffset] = prefix.deserialize(bytes, offset);
      offset = prefixOffset;
      if (isSome === 0) {
        return [umi.none(), fixed ? fixedOffset : offset];
      }
      const [value, newOffset] = item.deserialize(bytes, offset);
      offset = newOffset;
      return [umi.some(value), fixed ? fixedOffset : offset];
    }
  };
}

exports.option = option;
//# sourceMappingURL=option.cjs.map
